function [par_out,error_flag] = plot_grid_ind_debtox2019(filename)

% Usage: [par_out,error_flag] = plot_grid_ind_debtox2019(filename)
% 
% Function to combine MAT files for two chemicals (as generated by the
% parameter-space explorer) to make a MAT file for the mixture, assuming
% independent action. Independent action (in the strictest sense) implies
% that the two chemicals do not interact in any way. Therefore, combining
% multiple parameter clouds is simply a matter of randomly combining sets
% from all clouds. The only important thing is to make sure that the
% resulting cloud is not too large. This function only works properly if
% the MAT file was generated by the BYOM DEBtox2019 package for the simple
% compound models (incl. ERA_special)! Output is also meant for the simple
% compound DEBtox2019 mixture package.
% 
% Note: this is the function for engine, without using the parallel toolbox.
% 
% filename   cell array with strings for the name of MAT files to combine
% 
% Author     : Tjalling Jager
% Date       : September 2023
% Web support: <http://www.debtox.info/byom.html>

%  Copyright (c) 2012-2024, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

global glo2

SETTINGS_OPTIM = setup_settings(1); % load settings from file (use rough settings)

n_s = length(filename); % number of samples to compare
if n_s > 4 % for now, we only allow two files anyway, so this is for the future
    error_str = ('Cannot compare more than 4 samples at the moment');
    errordlg(error_str,'Error no. of sets'); % display a message box with the errors
    error_flag = 1; % signal main script that we ran into errors
    par_out    = [];
    return % simply stop as there is no useful input to work with
end

%% Run through the samples to load files and collect some things

% First, initialise matrices and cell arrays
BNDS  = cell(1,n_s);
COLL  = cell(1,n_s);
NAMES = cell(1,n_s);
BASIC = cell(1,n_s);
GLOCOLL = cell(1,n_s);

for i_s = 1:n_s % run through samples
    
    % Load sample <i_s>
    if exist([filename{i_s}],'file') == 2 % check if it exists first.
        load([filename{i_s}],'pmat','coll_all','GLO','names')
        % Load all of the saved information from the <calc_parspace> run.
    else % otherwise, produce an error (I don't think that is possible anymore)
        error(['There is no confidence set with filename ',filename{i_s},' in your working directory, so run calibration (with correct settings) first.'])
    end
    ind_basic = find(strcmp(names,'kd'))-1; % assume that kd is the first tox parameter!
    
    % Extract useful information from <pmat>.
    ind_fit  = find(pmat(:,2) == 1); % indices to fitted parameters (vector)
    pmat_lim = pmat(ind_fit,:); % limit <pmat_lim> to fitted parameters
    ind_log  = find(pmat_lim(:,5) == 0); % indices to log-scale parameters in fitted <pmat> (vector)
    
    coll_all(:,ind_log) = 10.^coll_all(:,ind_log); % put back on normal scale, where needed
    names_lim           = names(ind_fit); % only keep names for fitted parameters

    % put <pmat> on normal scale for log-parameters (this is needed for the
    % non-fitted parameters)
    pmat(pmat(:,5)==0,1) = 10.^(pmat(pmat(:,5)==0,1));

    % collect the results for further processing
    BNDS{i_s}  = pmat(ind_fit,[3 4]); % collect bounds used for original optimisation
    COLL{i_s}  = coll_all;
    NAMES{i_s} = names_lim;
    GLOCOLL{i_s}  = GLO;

    BASIC{i_s} = pmat(1:ind_basic,1); % collect value for basic parameters
    if any(pmat(1:ind_basic,2) == 1)
        warning('off','backtrace')
        warning(['Basic parameters were fitted in dataset ',filename{i_s},', this is ignored!'])
        warning('on','backtrace')
    end
    
end

% Chi2-criteria and indices to parameter sets to plot for inner and outer rim.
% We work here with the log-likelihood itself, so the chi2 criterion needs to be divided by 2.
chicrit_joint  = 0.5 * SETTINGS_OPTIM.crit_table(6,1); % criterion for joint 95% CI or parameters
chicrit_single = 0.5 * SETTINGS_OPTIM.crit_table(1,1); % criterion for single-parameter CIs
chicrit_prop   = 0.5 * SETTINGS_OPTIM.crit_prop(2);    % criterion for upper band of propagation band
chicrit_prop2  = 0.5 * SETTINGS_OPTIM.crit_prop(1);    % criterion for lower band of propagation band

%% Create a new <coll_all> matrix for use with mixtures

coll_all = []; % start with empty <coll_all> matrix

% find indices for all model parameters in names vector for each chemical
ind_kdA = find(strcmp(NAMES{1},'kd'));
ind_kdB = find(strcmp(NAMES{2},'kd'));
ind_zbA = find(strcmp(NAMES{1},'zb'));
ind_bbA = find(strcmp(NAMES{1},'bb'));
ind_zbB = find(strcmp(NAMES{2},'zb'));
ind_bbB = find(strcmp(NAMES{2},'bb'));

ind_zsA = find(strcmp(NAMES{1},'zs'));
ind_bsA = find(strcmp(NAMES{1},'bs'));
ind_zsB = find(strcmp(NAMES{2},'zs'));
ind_bsB = find(strcmp(NAMES{2},'bs'));

MLL = COLL{1}(1,end)+COLL{2}(1,end); % best minloglik is simply the sum of the first elements of the final column

% make sure best combination is in the <coll_add> matrix
coll_add = nan(1,11); % what we're adding to coll_all for set iA
coll_add(:,1) = COLL{1}(1,ind_kdA); % collect kdA
coll_add(:,2) = COLL{1}(1,ind_zbA); % collect zbA
coll_add(:,3) = COLL{1}(1,ind_bbA); % collect bbA
coll_add(:,4) = COLL{1}(1,ind_zsA); % collect zsA
coll_add(:,5) = COLL{1}(1,ind_bsA); % collect bsA

coll_add(:,6) = COLL{2}(1,ind_kdB); % collect kdB
coll_add(:,7) = COLL{2}(1,ind_zbB); % collect zbB
coll_add(:,8) = COLL{2}(1,ind_bbB); % collect bbB
coll_add(:,9) = COLL{2}(1,ind_zsB); % collect zsB
coll_add(:,10) = COLL{2}(1,ind_bsB); % collect bsB

coll_add(:,11) = COLL{1}(1,end)+COLL{2}(1,end); % new MLL

coll_all = cat(1,coll_all,coll_add); % collect in total <coll_all>
min_sz   = min(size(COLL{1},1),size(COLL{2},1)); % minimum size of the two clouds
ind_prop = 0;

f = waitbar(0,'Combining MAT files for DEBtox2019. Please wait.','Name','plot_grid_ind_debtox2019.m');
while ind_prop < 5000 % just a few rounds, until we have sufficient sets in the inner rim (incl. propagation set)
    % I aim here for at least 5000 elements in the inner rim of the new
    % cloud, which seems large enough.
    waitbar(min(1,ind_prop/5000),f); % update waiting bar

    % random permutations of parameter sets for A and B. <min_sz> elements
    indA = randperm(min_sz);
    indB = randperm(min_sz);

    coll_add = nan(min_sz,11); % what we're adding to <coll_all> in this round
    coll_add(:,1) = COLL{1}(indA,ind_kdA); % collect kdA
    coll_add(:,2) = COLL{1}(indA,ind_zbA); % collect zbA
    coll_add(:,3) = COLL{1}(indA,ind_bbA); % collect bbA
    coll_add(:,4) = COLL{1}(indA,ind_zsA); % collect zsA
    coll_add(:,5) = COLL{1}(indA,ind_bsA); % collect bsA

    coll_add(:,6) = COLL{2}(indB,ind_kdB); % collect kdB
    coll_add(:,7) = COLL{2}(indB,ind_zbB); % collect zbB
    coll_add(:,8) = COLL{2}(indB,ind_bbB); % collect bbB
    coll_add(:,9) = COLL{2}(indB,ind_zsB); % collect zsB
    coll_add(:,10) = COLL{2}(indB,ind_bsB); % collect bsB

    coll_add(:,11) = COLL{1}(indA,end)+COLL{2}(indB,end); % minloglik is sum of last columns

    coll_add(coll_add(:,11)>MLL+chicrit_joint,:) = []; % remove ones that are pretty bad

    coll_add  = sortrows(coll_add,size(coll_add,2)); % sort what we're adding based on minloglik
    ind_fin95 = find(coll_add(:,end) < coll_add(1,end) + chicrit_joint,1,'last'); % index to last element of sample that is still in joint CI
    ind_prop  = find(coll_add(:,end) < coll_add(1,end) + chicrit_prop,1,'last');  % index to last element of sample that is still in propagation band

    % we don't need so many in the outer rim, so we can downsample
    thin_sz  = ind_fin95-ind_prop; % elements in outer rim (outside propagation set)
    ind_thin = randperm(thin_sz,min(thin_sz,ind_prop)); % downsample to minimum of inner rim and outer rim
    % This means that you don't take more points from the outer rim
    % than there are in the inner. That seems to be okay.

    coll_all = cat(1,coll_all,coll_add(1:ind_prop,:),coll_add(ind_prop+ind_thin,:)); % collect in total <coll_all>
    coll_all = sortrows(coll_all,size(coll_all,2)); % sort again on minloglik
    ind_prop = find(coll_all(:,end) < coll_all(1,end) + chicrit_prop,1,'last'); % index to last element of sample that is still in propagation band

end

close(f) % close the waiting bar

coll_all = sortrows(coll_all,size(coll_all,2)); % sort again based on minloglik
% Find indices to sets within inner and outer rim.
ind_single = find(coll_all(:,end) < coll_all(1,end) + chicrit_single,1,'last'); % index to last element of sample that is still in inner rim
ind_fin95  = find(coll_all(:,end) < coll_all(1,end) + chicrit_joint,1,'last');  % index to last element of sample that is still in joint CI
ind_prop   = find(coll_all(:,end) < coll_all(1,end) + chicrit_prop,1,'last');   % index to last element of sample that is still in upper edge propagation band
ind_prop2  = find(coll_all(:,end) < coll_all(1,end) + chicrit_prop2,1,'last');  % index to last element of sample that is still in lower edge propagation band

%% Create new <pmat> for the mixture

% ind_kd = find(strcmp(names,'kd'));
glo2.names = cat(1,names(1:ind_basic),{'kdA';'zbA';'bbA';'zsA';'bsA';'kdB';'zbB';'bbB';'zsB';'bsB';'IAB'}); % names of the model parameters for new <coll_all> and <pmat>

pmat_rem = pmat;
pmat     = zeros(length(glo2.names),5); % create a new <pma>t for the mixture independent case

pmat(1:ind_basic,1) = mean([BASIC{1} BASIC{2}],2); % for basic parameters, take means from the data sets!
pmat(1:ind_basic,3:5) = pmat_rem(1:ind_basic,3:5); % this uses the columns 3:5 from the LAST set!

% find indices for all model parameters in new names vector for each chemical
ind_kdA2 = find(strcmp(glo2.names,'kdA'));
ind_zbA2  = find(strcmp(glo2.names,'zbA'));
ind_bbA2  = find(strcmp(glo2.names,'bbA'));
ind_zsA2  = find(strcmp(glo2.names,'zsA'));
ind_bsA2  = find(strcmp(glo2.names,'bsA'));

ind_kdB2  = find(strcmp(glo2.names,'kdB'));
ind_zbB2  = find(strcmp(glo2.names,'zbB'));
ind_bbB2  = find(strcmp(glo2.names,'bbB'));
ind_zsB2  = find(strcmp(glo2.names,'zsB'));
ind_bsB2  = find(strcmp(glo2.names,'bsB'));


% fill <pmat> with information from new <coll_all> 
pmat(ind_kdA2,:)  = [coll_all(1,1) 1 min(coll_all(:,1)) max(coll_all(:,1)) 0]; % kdA
pmat(ind_zbA2,:)  = [coll_all(1,2) 1 min(coll_all(:,2)) max(coll_all(:,2)) 0]; % zbA
pmat(ind_bbA2,:)  = [coll_all(1,3) 1 min(coll_all(:,3)) max(coll_all(:,3)) 0]; % bbA
pmat(ind_zsA2,:)  = [coll_all(1,4) 1 min(coll_all(:,4)) max(coll_all(:,4)) 0]; % zsA
pmat(ind_bsA2,:)  = [coll_all(1,5) 1 min(coll_all(:,5)) max(coll_all(:,5)) 0]; % bsA

pmat(ind_kdB2,:)  = [coll_all(1,6) 1 min(coll_all(:,6)) max(coll_all(:,6)) 0]; % kdB
pmat(ind_zbB2,:)  = [coll_all(1,7) 1 min(coll_all(:,7)) max(coll_all(:,7)) 0]; % zbB
pmat(ind_bbB2,:)  = [coll_all(1,8) 1 min(coll_all(:,8)) max(coll_all(:,8)) 0]; % bbB
pmat(ind_zsB2,:)  = [coll_all(1,9) 1 min(coll_all(:,9)) max(coll_all(:,9)) 0]; % zsB
pmat(ind_bsB2,:)  = [coll_all(1,10) 1 min(coll_all(:,10)) max(coll_all(:,10)) 0]; % bsB

pmat(ind_bsB2+1,:) = [0     0 -100  100 1]; % IAB

% Make the bounds a bit wider, but not wider than the original bounds in the saved pmat
bnds_extra = 1.5; % what to divide/multiply min-max bounds with to make them a bit wider than sample

pmat([ind_kdA2 ind_kdB2],3) = max(pmat([ind_kdA2 ind_kdB2],3)/bnds_extra,[BNDS{1}(ind_kdA,1);BNDS{2}(ind_kdB,1)]); % min bounds for kdA and kdB
pmat([ind_kdA2 ind_kdB2],4) = min(pmat([ind_kdA2 ind_kdB2],4)*bnds_extra,[BNDS{1}(ind_kdA,2);BNDS{2}(ind_kdB,2)]); % max bounds for kdA and kdB
pmat([ind_zbA2 ind_zbB2],3) = max(pmat([ind_zbA2 ind_zbB2],3)/bnds_extra,[BNDS{1}(ind_zbA,1);BNDS{2}(ind_zbB,1)]); % min bounds for zbA and zbB
pmat([ind_zbA2 ind_zbB2],4) = min(pmat([ind_zbA2 ind_zbB2],4)*bnds_extra,[BNDS{1}(ind_zbA,2);BNDS{2}(ind_zbB,2)]); % max bounds for zbA and zbB
pmat([ind_bbA2 ind_bbB2],3) = max(pmat([ind_bbA2 ind_bbB2],3)/bnds_extra,[BNDS{1}(ind_bbA,1);BNDS{2}(ind_bbB,1)]); % min bounds for bbA and bbB
pmat([ind_bbA2 ind_bbB2],4) = min(pmat([ind_bbA2 ind_bbB2],4)*bnds_extra,[BNDS{1}(ind_bbA,2);BNDS{2}(ind_bbB,2)]); % max bounds for bbA and bbB
pmat([ind_zsA2 ind_zsB2],3) = max(pmat([ind_zsA2 ind_zsB2],3)/bnds_extra,[BNDS{1}(ind_zsA,1);BNDS{2}(ind_zsB,1)]); % min bounds for zsA and zsB
pmat([ind_zsA2 ind_zsB2],4) = min(pmat([ind_zsA2 ind_zsB2],4)*bnds_extra,[BNDS{1}(ind_zsA,2);BNDS{2}(ind_zsB,2)]); % max bounds for zsA and zsB
pmat([ind_bsA2 ind_bsB2],3) = max(pmat([ind_bsA2 ind_bsB2],3)/bnds_extra,[BNDS{1}(ind_bsA,1);BNDS{2}(ind_bsB,1)]); % min bounds for bsA and bsB
pmat([ind_bsA2 ind_bsB2],4) = min(pmat([ind_bsA2 ind_bsB2],4)*bnds_extra,[BNDS{1}(ind_bsA,2);BNDS{2}(ind_bsB,2)]); % max bounds for bsA and bsB

ind_fit = find(pmat(:,2) == 1); % indices to fitted parameters (vector)

% bounds that are smaller than a factor 10 can be fitted on normal scale
chk_bnds = pmat(:,2) == 1 & (pmat(:,4)./pmat(:,3)) < 10; % which elements are smaller than 10?
pmat(chk_bnds,5) = 1; % put them to normal scale

%% Display, plot and final things

% display par in a formatted way so they can be directly copied into the code of the main script
par_out = packunpack(2,[],pmat);
disp(' ')
disp('  Independent action for DEBtox2019')
disp('You can copy-paste following lines into script for mixture predictions')
disp('if you like to run the parameter-space explorer on the mixture data.')
print_par(par_out) % this prints out the optimised parameter values in a

pmat_lim = pmat(ind_fit,:);
ind_log  = find(pmat_lim(:,5) == 0); % indices to log-scale parameters in fitted <pmat> (vector)

% improvise a <pmat_print>, though without any CIs, so it can be saved
pmat_print = zeros(length(ind_fit),8);
pmat_print(:,1) = pmat_lim(:,1); % copy best-fit values
% add CIs for the model parameters, estimated as the edges of the
% propagation set; this is a good approximation
pmat_print(:,2) = (min(coll_all(1:ind_prop,1:end-1),[],1))';
pmat_print(:,3) = (max(coll_all(1:ind_prop,1:end-1),[],1))';

disp(' ')
disp('=================================================================================')
disp('Results of the parameter-space combination for mixture predictions')
disp('=================================================================================')
disp('Independent action for DEBtox2019')
disp(['   Chemical A is taken from: ',filename{1}])
disp(['   Chemical B is taken from: ',filename{2}])
disp(['   Sample: ',num2str(ind_fin95),' sets in joint CI and ',num2str(ind_single),' in inner CI.'])
disp(['   Propagation set: ',num2str(1+ind_prop - ind_prop2),' sets will be used for error propagation.'])

FVAL = coll_all(1,end);
fprintf('   Minus log-likelihood has reached the value %#1.2f (AIC=%#1.2f). \n',FVAL,2*length(ind_fit)+2*FVAL)

disp('Approx. best estimates and 95% CIs on fitted parameters')
disp_pmat(pmat,pmat_print); % call dedicated function for printing the <pmat>
disp('Confidence intervals are estimated from the sample (inner rim, incl. propagation band)')
disp('so they will generally slightly exaggerate the true intervals.')

% put <pmat> on log-scale for log-parameters (this is how BYOM uses it)
% pmat(pmat(:,2)==1 & pmat(:,5)==0,1) = log10(pmat(pmat(:,2)==1 & pmat(:,5)==0,1));
pmat(pmat(:,5)==0,1) = log10(pmat(pmat(:,5)==0,1)); % for ALL parameters

% put <coll_all> on log-scale for log-parameters
coll_all(:,ind_log) = log10(coll_all(:,ind_log));

coll_prof_pruned = cell(1,length(ind_fit));
for i_p = 1:length(ind_fit) % trick to make sure profiles are plotted (but without refinement red line)
    coll_prof_pruned{i_p} = [NaN NaN];
end

% And make a plot of the final results as parameter-space plot
figh = plot_grid(pmat,[],coll_all,coll_prof_pruned,[],SETTINGS_OPTIM); 

% user can give a filename for saving, but I try to provide a good default
filenm = ['IND_',filename{1}(13:end-26),'+',filename{2}(13:end-26),'_PS.mat'];

% % Instead of the line above, feel free to uncomment the code below to use
% % the Matlab GUI for entering a filename for saving.
% filenm = uiputfile(['IND_',filename{1}(11:end-12),'+',filename{2}(11:end)],'Save new project file for mixture of chemical A and B'); % use Matlab GUI to select name for saving MAT file
% if numel(filenm) == 1 && filenm(1) == 0 % if cancel is pressed ...
%     error_flag = 1; % signal main script that we ran into errors
%     return % simply stop as the user does not want to save
% end

saveplt = 0; % for now, don't save (since this function is not normally called from a standard byom script, the glo.saveplot will not be defined)
figure(figh)  % use existing handle and make sure that the multiplot is the current plot
% Add a title to the plot.
axes('Position',[0 0 1 1],'Xlim',[0 1],'Ylim',[0 1],'Box','off','Visible','off','Units','normalized', 'clipping','off');
text(0.5, 1,['File: ',filenm,' (',date,')'],'Interpreter','none','HorizontalAlignment','center','VerticalAlignment', 'top');
if saveplt > 0
    save_plot(figh,['parspace_indmixpred_',glo.basenm]) % save parspace plot in output folder
end
snapnow % for publishing Matlab scripts: make plot appear in the context of the screen output

GLO.basenm = filenm(1:end-4);
GLO.moa   = [GLOCOLL{1}.moa;GLOCOLL{2}.moa];
GLO.feedb = [GLOCOLL{1}.feedb;GLOCOLL{2}.feedb];

% save new mat file
par   = par_out;
names = glo2.names ;
save(filenm,'par','pmat','coll_all','pmat_print','coll_prof_pruned','names','GLO')
