function [MinColl,MinCI,ind_traits] = calc_effect_window(par_plot,fname_prof,Twin,opt_ecx,opt_conf)
 
%  Syntax: [MinColl,MinCI,ind_traits] = calc_effect_window(par_plot,fname_prof,Twin,opt_ecx,opt_conf)
%           PART OF ENGINE_PAR
% Calculate toxic effects due to an exposure profile, with moving time
% window and different (fixed) multiplication factors. This function should
% work with every TKTD model you throw at it, as long as there is at least
% one state variables indicated with one of the dedicated traits (whose
% position in the state vector is given by: <glo.locS>, <glo.locL> or
% <glo.locR>).
% 
% Some calculation speed can be gained by adding an option that skips
% recalculation of the control response when running through a sample for
% CIs, and when running through different start points for the time window
% (at least, when only tox parameters are fitted).
% 
% <par_plot>   parameter structure for the best-fit curve; if left empty the
%              structure from the saved sample is used
% <fname_prof> filename for the file containing the exposure profile
% <Twin>       length of time window (days)
% <opt_ecx>    options structure for ECx and EPx calculations
% <opt_conf>   options structure for making confidence intervals
% 
% <MinColl> collects, for each state, the MF, minimum of the trait relative
% to the control (thus largest effect), and time at which it occurs.
% <MinCI> collects the CI for the minimum trait level. Structure is
% MinColl{i_MF}(i_X,:). Output of ind_traits is needed to know which state
% is meant with i_X.
% 
% Author     : Tjalling Jager 
% Date       : May 2023
% Web support: http://www.debtox.info/byom.html

%  Copyright (c) 2012-2024, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

global glo glo2 X0mat

WRAP.glo  = glo;
WRAP.glo2 = glo2;
% Note that glo will be changed in this function. That does not affect the
% functioning of WRAP, since WRAP is only used for packunpack here.

names     = glo2.names;
filenm    = glo.basenm;
glo_rem   = glo; % remember global glo before we change it

backhaz   = opt_ecx.backhaz;   % parameter name (as string) to set to zero for LCx/LPx calculation to remove background mortality
setzero   = opt_ecx.setzero;   % parameter names (as string array) for extra paramaters to be set to zero
X_excl    = opt_ecx.statsup;   % states to suppress from the calculations (e.g., locS)
MF        = opt_ecx.mf_range;  % range for MFs to make plots
par_read  = opt_ecx.par_read;  % when set to 1 read parameters from saved set, but do NOT make CIs
batch_epx = opt_ecx.batch_epx; % when set to 1 use batch mode (no output to screen)
notitle   = opt_ecx.notitle;   % set to 1 to suppress titles above plots
start_neg = opt_ecx.start_neg; % set to 1 to start the moving window at minus window width
calc_int  = opt_ecx.calc_int;  % integrate survival and repro into 1) RGR, or 2) survival-corrected repro (experimental!)
prune_win = opt_ecx.prune_win; % set to 1 to prune the windows to keep the interesting ones
Tstep     = opt_ecx.Tstep;     % stepsize or resolution of the time window (default 1 day)
id_sel    = opt_ecx.id_sel; % scenario to use from X0mat, scenario identifier, flag for ECx to use scenarios rather than concentrations

N_t    = 20; % number of time points for regular effect window
N_tint = 200; % number of time points for when we integrate survival and repro
% For a FOCUS profile there should be no need to specify a detailed time
% vector as the ODE solver will dictate the time step, and we only need the
% end result. In any case, if more calculation detail is needed, that
% should be dealt with in call_deri and not here. Since the calculation in
% this function does not use any shortcuts, there is no need to have more
% detail here. Exception is when we want to integrate survival and repro
% over time (into an intrinsic rate or survival-corrected reproduction).
% For long windows (large Twin), it would be good to increase N_tint. Note:
% for truly pulsed exposure, make sure to break up the time vector in
% call_deri.

if isempty(opt_conf)
    type_conf = 0; % then we don't need CIs
    use_par_out = 0; % set to 1 to use par_out, as entered in this function, rather than from saved set
else
    type_conf = opt_conf.type; % use values from slice sampler (1), likelihood region(2) to make intervals
    type_conf = max(0,type_conf); % if someone uses -1, set it to zero
    use_par_out = opt_conf.use_par_out; % set to 1 to use par_out, as entered in this function, rather than from saved set
end

if batch_epx == 0 % don't show warnings if we're in batch mode
    diary(glo.diary) % collect output in the diary "results.out"
    warning('off','backtrace')
    if ~isempty(glo.names_sep)
        warning('You are using separate parameters per data set (with glo.names_sep)')
        warning(['Parameters for data set ',num2str(1+floor(id_sel(2)/100)),' will be used for effects'])
        disp(' ')
    end
    if isfield(glo,'moa') && isfield(glo,'feedb')
        if glo.feedb(2) == 1 && any(glo.moa(1:3)==1)
            warning('Calculating effect windows at fixed MFs is NOT recommended for this combination of pMoA and feedbacks!')
            warning('There is a possibility of (limited) higher effects at lower MFs, so use extra care.')
            disp(' ')
        end
    end
    disp(' '), warning('on','backtrace')
    diary off
end

MinCI = []; % make sure it is defined, even when no CIs are made

glo.scen_plot = 0; % do not make a plot when calling make_scen

% see which other states are there
locS = [];
locL = [];
locR = [];
if isfield(glo,'locS') && ~ismember(glo.locS,X_excl) % then we have a state of survival
    locS = glo.locS; % collect the location
end
if isfield(glo,'locL') && ~ismember(glo.locL,X_excl) % then we have a state of body length
    locL = glo.locL; % collect the location
end
if isfield(glo,'locR') && ~ismember(glo.locR,X_excl) % then we have a state of reproduction
    locR = glo.locR; % collect the location
end
% And also add the states for the GUTS immobility package. For now, active
% only, since for that trait, it is easy to calculate ECx relative to the
% control (for death and immobile, the control is zero). There is a way to
% calculate EPx for death, but that would require summing active and
% immobile animals before calculating the effect (or take 1-death), so that
% is a bit more work.
loc_a = [];
if isfield(glo,'loc_a') % then we have a state of active
    loc_a = glo.loc_a; % collect the locations
end

ind_traits = [locS locL locR loc_a]; % indices for the traits we want from Xout
% We seem to have no interest for damage ...

% vector with initial values for the states in the simulations
loc_id = find(X0mat(1,:) == id_sel(1)); % find where specified ID is in X0mat
if ~isempty(loc_id)
    X0mat_tmp = X0mat(:,loc_id); % take correct column for our analysis
else % if we cannot find the specified ID ...
    X0mat_tmp = X0mat(:,1); % take first column for our analysis
end
X0mat_tmp(1) = id_sel(2); % give the scenario specified name

% If we need CIs, load the best parameter set and the random sample from file
if type_conf > 0 || isempty(par_plot) % also if par_plot is not provided
    [rnd,par] = load_rnd(opt_conf); % loading and selecting the sample is handled in a separate function
    if numel(rnd) <= 1 % that means that no (useful) sample was found
        type_conf = 0; % no need to produce an error, just do analysis without CI
    end
    if isempty(par_plot) % if no par structure was entered in this function ...
        par_plot = par; % simply use the one from the sample file
    end
    if type_conf < 1 || par_read == 1 % then we don't want to make CIs
        type_conf = 0;  % don't make CIs anymore (when triggered by par_read)
        rnd       = []; % make sample empty
    end
end

% if ~isfield(par_plot,'tag_fitted') % apparently, parameters have not been fitted
%     warning('off','backtrace')
%     warning('You did not fit any parameters, so LCx or LPx is based on the values in the initial parameter matrix par.')
%     warning('Any CIs are made from the saved set in the MAT file.')
%     disp(' '), warning('on','backtrace')
% end

% backhaz is by default set to 'hb' in prelim_checks.m for the GUTS package
% identify background hazard and set it to zero
if isempty(backhaz) || ~isfield(par_plot,backhaz) % we need to make background mortality zero
    error('The function calc_effect_window expects a parameter name in opt_ecx.backhaz, which matches a parameter name in your parameter structure that can be set zero to remove background mortality.')
else
    par_plot.(backhaz)(1) = 0; % set parameter to zero in par_plot
    loc_zero = strcmp(names,backhaz)==1; % where is this parameter in the par structure? (logical indexing)
end
% allow extra parameters to be set to zero, such as initial concentrations
if ~isempty(setzero)
    if ~iscell(setzero) % just to make sure it will be a cell
        setzero = {setzero}; % turn it into a cell array with 1 element
    end
    for i = 1:length(setzero)
        par_plot.(setzero{i})(1) = 0; % set parameter to zero in par_plot
        loc_zero = loc_zero == 1 | strcmp(names,setzero{i})==1; % add this parameter to loc_zero (logical indexing)
    end
end

% Load exposure profile from file to use with linear interpolation
Cw     = load(fname_prof);
Tend   = Cw(end,1); % last time point in profile

if isempty(Twin) % if Twin is left empty, we just calculate the entire profile
    Trange = 0;
    Twin   = Tend; % last time point in profile
    indT   = [1 length(Cw)]; % indices for the start and stop point in Cw
    t      = linspace(0,Tend,N_t);
else
    % Prepare exposure profile for moving-time window calculations
    [Cw,indT,Trange] = prep_exp_profile(Cw,Twin,Tstep,start_neg);
    t = linspace(0,Twin,N_t); 
end
Tstrt = Cw(1,1);

% ================== TEST ======================
WRAP2 = [];
if calc_int > 0 % when calculating integration of survival and reproduction ...
    t  = linspace(0,t(end),N_tint); % take more-detailed time vector as we need to integrate
    t  = t(:); % make sure it is a column vector
    t2 = mean([t(1:end-1) t(2:end)],2); % new averaged time vector
    WRAP2.t  = t;
    WRAP2.t2 = t2;
    WRAP2.Th = glo.Tbp; % hatching time is brood-pouch delay (if any)
    WRAP2.rgr_init = 1; % initial guess for the growth rate
    % last 3 lines are for intrinsic rate only, but do not hurt for
    % survival-corrected repro
    
    if ~isfield(glo,'locS')
        glo.locS = [];
    end
    if ~isfield(glo,'locR')
        glo.locR = [];
    end
end
% ================== TEST ======================

% =================== TEST ================================================
if prune_win == 1
    Trange_sel = prune_windows(Trange,Cw,Twin); % prune the range to the interesting windows
end
% =================== TEST ================================================

% figure
% plot(Cw(:,1),Cw(:,2),'k-')

if type_conf > 0 % if we make CIs ...
    
    if use_par_out == 1
        par = par_plot; % then we'll use the input par, rather than the saved one
        % Note: par_plot must already be structured in the main script, such
        % that the fitted parameters match the ones in the saved set, etc.
        % Note: when par_plot is NOT entered, it will have been made equal
        % to par from the saved set.
    elseif batch_epx == 0
        par_comp(par,par_plot,cat(2,backhaz,setzero)) % compare par from input with the one from the MAT file
    end
    
    n_sets   = size(rnd,1); % number of samples from parameter space
    pmat     = packunpack(1,par,0,WRAP); % transform structure *from saved set* into a regular matrix
    % it is better to use the saved par, as there may be differences in the
    % log-setting of parameters between the saved set and the optimised
    % par_out matrix (especially when using the alllog option in
    % calc_slice).
    
    ind_fit    = (pmat(:,2)==1); % indices to fitted parameters
    ind_logfit = (pmat(:,5)==0 & pmat(:,2)==1); % indices to pars on log scale that are also fitted!
end

if batch_epx == 0 % don't show waitbar if we're in batch mode
    if type_conf > 0 % if we make CIs ...
        f = waitbar(0,'Calculating moving time window with various MFs and CIs. Please wait.','Name','calc_effect_window.m');
    else
        f = waitbar(0,'Calculating moving time window with various MFs. Please wait.','Name','calc_effect_window.m');
    end
end

N_traits = length(ind_traits);
if calc_int > 0 % if we integrate survival and repro ...
    N_traits = 1; % we only have 1 trait left
end

% Xcoll will collect output, Xlo/Xhi the CIs, initialised with NaNs
Xcoll = cell(1,length(MF));
Xlo   = cell(1,length(MF));
Xhi   = cell(1,length(MF));
for i = 1:length(MF) % run through standard multiplication factors
    Xcoll{i} = nan(length(Trange),1+N_traits);
    Xlo{i}   = nan(length(Trange),N_traits);
    Xhi{i}   = nan(length(Trange),N_traits);
end

% Start/check parallel pool
% That is always needed here since I also use the parallel toolbox now for
% the different MFs, even without CIs! This may change if we're going to do
% batch processing.
if glo2.n_cores > 0
    poolobj = gcp('nocreate'); % get info on current pool, but don't create one just yet
    if isempty(poolobj) % if there is no parallel pool ...
        parpool('local',glo2.n_cores) % create a local one with specified number of cores
    end
end

for i_T = 1:length(Trange) % run through all time points (start of window)
    
    if batch_epx == 0
        waitbar(i_T/length(Trange),f) % update the waiting bar
    end
    
    if prune_win == 1 && Trange_sel(i_T) == 0 % then this window is pruned
        continue % so move to next window!
    end
    
    T      = [Trange(i_T);Trange(i_T)+Twin]; % time window of length Twin
    Cw_tmp = Cw(indT(i_T,1):indT(i_T,2),:);  % extract only the profile that covers the time window
    if Cw_tmp(end,1)-Cw_tmp(1,1) ~= Twin     % to make absolutely sure ...
        error('Something went wrong: the exposure window length does not match the required length.')
    end
    
    Cw_tmp(:,1) = Cw_tmp(:,1)-T(1);     % make time vector for the short profile start at zero again
    Cw_tmp      = [1 id_sel(2);Cw_tmp]; % add a first row with a scenario identifier
    
    make_scen(-5,-1);    % remove all spline info
    make_scen(4,Cw_tmp); % create the globals to define the forcing function
    
    % First calculate the control response in this time window. It is
    % usually superfluous to do this for each start point of the time
    % window: since there is no exposure in the control, the response will
    % be the same in each window. However, somebody might implement
    % time-varying temperatures or food conditions ...
    
    [~,Xctrl] = calc_epx_helper(0,calc_int,t,par_plot,X0mat_tmp,glo,ones(1,N_traits),ind_traits,[],WRAP2);
    % Note: modifying X0mat_tmp is more efficient than setting glo.MF=0, as
    % in that case, call_deri will still treat it as a time-varying
    % exposure, and run through it in steps.
    if calc_int == 1
        WRAP2.rgr_init = [0 1.2*Xctrl]; % update initial guess (new one will not be far from old one)
    end
    
    glo_tmp   = glo; % copy the global to get parfor running ...

    parfor i_MF = 1:length(MF) % run through standard multiplication factors
        [~,Xout] = calc_epx_helper(MF(i_MF),calc_int,t,par_plot,X0mat_tmp,glo_tmp,Xctrl,ind_traits,[],WRAP2);
        Xcoll{i_MF}(i_T,:) = [MF(i_MF) Xout]; % collect the effect relative to the control
    end
    
    if type_conf > 0 % if we want CIs ... we'll do it again for each element of the sample
        
        % create a huge matrix to catch effect levels, for every set and every case
        Xcoll_tmp = nan(n_sets,length(MF),N_traits);
        
        % some trickery to get parfor running ... First create a pmat_coll with
        % all sets from the sample!
        pmat_coll = cell(n_sets,1); % cell array to construct pmat for each sample
        for k = 1:n_sets % run through all sets in the sample
            pmat(ind_fit,1) = rnd(k,:); % replace values in pmat with the k-th random sample from the MCMC
            % put parameters that need to be fitted on log scale back on normal
            % scale (as call_deri requires normal scale, in contrast to transfer.m)
            if sum(ind_logfit)>0
                pmat(ind_logfit,1) = 10.^(pmat(ind_logfit,1));
            end
            % Note: pmat is still on normal scale here, but the sample in rnd
            % contains the value on a log scale, if a parameter is fitted on log
            % scale. Call_deri needs normal scale structure par_k.
            pmat(loc_zero,1) = 0; % make selected parameter(s) zero in each set of the sample!
            % this has to be done after the tranformation to normal scale!
            pmat_coll{k} = pmat; % collect it in a huge cell array
        end
        
        % some trickery to get parfor running ...
        glo_tmp   = glo;
        L_i_MF    = length(MF);
        
        parfor k = 1:n_sets % run through all sets in the sample
            
            par_k = packunpack(2,0,pmat_coll{k},WRAP); % transform parameter matrix into a structure
            
            % Calculate the control response for this parameter set. When only
            % the tox parameters are fitted, this is superfluous. However, we
            % should not make a priori assumptions about how this function will
            % be used! E.g., for GUTS cases, hb may be fitted as well, and we
            % need to have the effect relative to the control for THIS set of
            % the sample.
            
            WRAP3 = WRAP2; % this is needed to please parfor
            WRAP3.rgr_init = 1; % reset initial guess for rgr
            [~,Xctrl] = calc_epx_helper(0,calc_int,t,par_k,X0mat_tmp,glo_tmp,ones(1,N_traits),ind_traits,[],WRAP3);
            if calc_int == 1
                WRAP3.rgr_init = [0 1.2*Xctrl]; % update initial guess (new one will not be far from old one)
            end
            
            % than go through the multiplication factors
            for i_MF = 1:L_i_MF % run through standard multiplication factors
                [~,X_tmp] = calc_epx_helper(MF(i_MF),calc_int,t,par_k,X0mat_tmp,glo_tmp,Xctrl,ind_traits,[],WRAP3);
                % use of a sub-function is also needed to get parfor to cooperate
                Xcoll_tmp(k,i_MF,:) = X_tmp; % collect the answer!
            end
        end
        
        % Now find the boundaries of the CIs
        for i_X = 1:N_traits % run through traits
            for i_MF = 1:length(MF) % run through standard multiplication factors
                if type_conf == 1 % then we're doing Bayes
                    Xlo{i_MF}(i_T,i_X) = prctile(Xcoll_tmp(:,i_MF,i_X),2.5,1);
                    Xhi{i_MF}(i_T,i_X) = prctile(Xcoll_tmp(:,i_MF,i_X),97.5,1);
                else % take min-max
                    Xlo{i_MF}(i_T,i_X) = min(Xcoll_tmp(:,i_MF,i_X),[],1);
                    Xhi{i_MF}(i_T,i_X) = max(Xcoll_tmp(:,i_MF,i_X),[],1);
                end
            end
        end
        clear Xcoll_tmp % no need for this large matrix anymore
        
    end
end

if batch_epx == 0
    close(f) % close the waiting bar
end

if calc_int > 0
    ind_traits = -1;
end

%% Find where the largest effect occurs and how large it is

MinColl = cell(1,length(MF));
MinCI   = cell(1,length(MF));
for i_X = 1:N_traits % run through traits
    for i_MF = 1:length(MF) % run through standard multiplication factors
        % find largest effect, and time window where it occurs, for each
        % state at each MF; note that I ignore effects less than 1
        % permille, and round effects more than 99.9% to 100%
        [val_min,ind_min] = min(Xcoll{i_MF}(:,i_X+1));
        if val_min > 0.999 % avoid apparent effects due to numerical inaccuracy
            val_min = 1;
            ind_min = 1;
        end
        if val_min < 0.001 % avoid extreme minima which make little sense
            val_min = 0;
        end
        MinColl{i_MF}(i_X,:) = [MF(i_MF) val_min Trange(ind_min)]; % collect MF, minimum and time at which it occurs
        MinCI{i_MF}(i_X,:)   = [Xlo{i_MF}(ind_min,i_X) Xhi{i_MF}(ind_min,i_X)]; % also collect CI on the minimum
    end
end

if batch_epx == 1  % in batch mode, we can stop here
    glo = glo_rem; % return glo to its original value
    return 
end

%% Plot the results
% This makes a multiplot: the different endpoints (traits) are shown in
% rows (plotted as response relative to the control), and the different MFs
% in columns. Note that the x-axis is the time at which the time window
% starts; the window is shifted across the entire profile, until it extends
% beyond the profile.

n = length(MF);
m = N_traits;
[figh,ft] = make_fig(m,n,2); % create a figure window of correct size

for i_X = 1:N_traits % run through traits
    
    for i_MF = 1:length(MF) % run through standard multiplication factors
        
        h_pl = subplot(m,n,(i_X-1)*length(MF)+i_MF); % make a sub-plot
        hold on
        h_ax = gca;
        set(h_ax,'LineWidth',1,ft.name,ft.ticks) % adapt axis formatting
        if m>1 && n>1 % only shrink white space when there are more than 1 rows and columns
            p = get(h_pl,'position');
            p([3 4]) = p([3 4])*1.1; % add 10 percent to width and height
            set(h_pl, 'position', p);
        end
        
        % create axis labels
        if i_X == length(ind_traits) % only x-label at last row
            xlab = ['start ' glo.xlab]; % label
            h_tmp = xlabel(xlab); set(h_tmp,ft.name,ft.label)
        else
            set(h_ax,'XTickLabel',[]); % remove tick labels on x-axis
        end
        if i_MF == 1 % only y-label in first column
            switch calc_int
                case 0
                    ylab = ['rel. ' glo.ylab{ind_traits(i_X)}]; % label
                case 1
                    ylab = 'relative intrinsic rate';
                case 2
                    ylab = 'survival-corrected repro';
            end
            h_tmp = ylabel(ylab); set(h_tmp,ft.name,ft.label)
        else
            set(h_ax,'YTickLabel',[]); % remove tick labels on y-axis
        end
        if i_X == 1 % only title in first row
            title(['MF = ',num2str(MF(i_MF))])
        end
        
        if length(Trange) == 1 % have to come up with something special for when there is only 1 time point
            
            % also plot critical lines for 10 and 50% effect
            plot([-1 1],[0.9 0.9],'k--')
            plot([-1 1],[0.5 0.5],'k--')
            
            errorbar(0,Xcoll{i_MF}(:,i_X+1),Xcoll{i_MF}(:,i_X+1)-Xlo{i_MF}(:,i_X),Xhi{i_MF}(:,i_X)-Xcoll{i_MF}(:,i_X+1),'ko-','MarkerFaceColor','k','LineWidth',1)
            ylim([0 1.05]) % limit y-axis
            xlim([-1 1]) % limit x-axis
            
        else
            
            if type_conf > 0 % then we have CIs to plot
                % Little trick to fill the area between the two curves, to
                % obtain a coloured confidence interval as a band.
                t2  = [Trange';flipud(Trange')]; % make a new time vector that is old one, plus flipped one
                Xin = [Xlo{i_MF}(:,i_X);flipud(Xhi{i_MF}(:,i_X))]; % do the same for the plot line, hi and lo
                fill(t2(~isnan(Xin)),Xin(~isnan(Xin)),'g','LineStyle','none','FaceAlpha',1) % and fill this object
            end
            
            plot(Trange,Xcoll{i_MF}(:,i_X+1),'k-','LineWidth',1) % plot effects relative to control
            ylim([0 1.05]) % limit y-axis
            xlim([Tstrt Tend]) % limit x-axis
            
            if type_conf > 0 % then we have CIs to plot
                plot(Trange,Xlo{i_MF}(:,i_X),'k:') % plot effects
                plot(Trange,Xhi{i_MF}(:,i_X),'k:') % plot effects
            end
            
            % also plot critical lines for 10 and 50% effect
            plot([Tstrt Tend],[0.9 0.9],'k--')
            plot([Tstrt Tend],[0.5 0.5],'k--')
            plot([0 0],[0 1.05],'k:') % and for start of profile
        end
        
%         % find largest effect, and time window where it occurs, for each
%         % state at each MF
%         [val_min,ind_min] = min(Xcoll{i_MF}(:,i_X+1));
%         if val_min > 0.999 % avoid apparant minima due to numerical inaccuracy
%             val_min = 1;
%             ind_min = 1;
%         end
%         if val_min < 0.001 % avoid extreme minima which make little sense
%             val_min = 0;
%         end
%         MinColl{i_MF}(i_X,:) = [MF(i_MF) val_min Trange(ind_min)]; % collect MF, minimum and time at which it occurs
%         MinCI{i_MF}(i_X,:)   = [Xlo{i_MF}(ind_min,i_X) Xhi{i_MF}(ind_min,i_X)]; % also collect CI on the minimum
    end
end

if notitle == 0
    switch type_conf
        case 0
            tit = 'No confidence intervals';
        case 1
            tit = 'CIs: Bayesian 95% credible interval';
        case 2
            tit = 'CIs: 95% pred. likelihood, shooting method';
        case 3
            tit = 'CIs: 95% pred. likelihood, parspace explorer';
    end
    axes('Position',[0 0 1 1],'Xlim',[0 1],'Ylim',[0 1],'Box','off','Visible','off','Units','normalized', 'clipping','off');
    text(0.5, 1,tit,'HorizontalAlignment','center','VerticalAlignment', 'top');
end

drawnow

if glo.saveplt > 0 % if we want to save the plot
    savenm = ['effect_window_plot_',filenm];%
    save_plot(figh,savenm);
end

%% Print results on screen

% We might be called with an entire path to a file, rather than just a
% filename. In that case, only display the filename with the lowest folder
% it is in.
a = strfind(fname_prof,filesep);
if length(a) > 2
    fname_prof = fname_prof(a(end-1):end);
end
    
diary(glo.diary) % collect output in the diary "results.out"
% disp(' ')
disp('Results from moving time window calculations')
disp(['  Exposure from file: ',fname_prof])
disp(['  Length of window  : ',num2str(Twin)])
disp(['  stepsize window   : ',num2str(Tstep)])
switch type_conf
    case 0
        disp('  Largest effect without confidence intervals');
    case 1
        disp('  Largest effect with CIs: Bayesian 95% credible interval');
    case 2
        disp('  Largest effect with CIs: 95% pred. likelihood, shooting method');
    case 3
        disp('  Largest effect with CIs: 95% pred. likelihood, parspace explorer');
end
disp('================================================================================')
disp('  Most sensitive time window for various traits')
disp('  Note that multiple minima may occur; only first one is indicated')
disp('  Also note that most sensitive window may depend on the MF')
disp('  Effect is given as relative value of trait, so small values imply large effect')
if type_conf > 0
    disp('  CI is for the window with the lowest EPx, so not necessarily the lowest overall')
end
disp('================================================================================')
for i_MF = 1:length(MF) % run through standard multiplication factors
    disp(['MF = ',num2str(MF(i_MF))])
    for i_X = 1:N_traits
        if calc_int > 0
            if calc_int == 1
                fprintf('  Intrinisic rate: ')
            else
                fprintf('  Surv-corr repro: ')
            end
        else
            switch ind_traits(i_X)
                case locS
                    fprintf('  Survival    : ')
                case locL
                    fprintf('  Body length : ')
                case locR
                    fprintf('  Reproduction: ')
                case loc_a
                    fprintf('  Active      : ')
            end
        end
                
        if MinColl{i_MF}(i_X,2) > 0.999
            fprintf('no effects \n')
        else
            % assume that time window always starts on a full day (the %3.0f
            % only prints whole numbers)
            fprintf('%#10.2g at t = %3.1f ',MinColl{i_MF}(i_X,2),MinColl{i_MF}(i_X,3))
            fprintf('(%#1.2g - %#1.2g) \n',MinCI{i_MF}(i_X,1),MinCI{i_MF}(i_X,2))
        end
    end
end
disp('================================================================================')
disp(['Time required: ' secs2hms(toc)])
disp('')
diary off

% minimum window for repro across MFs
MinRep = [];
for i_MF = 1:length(MF) % run through standard multiplication factors
    for i_X = 1:N_traits
        if ind_traits(i_X) == locR % then we have repro data
            % collect MF, relative cumul. repro, and start of time window
            MinRep = cat(1,MinRep,[MF(i_MF) MinColl{i_MF}(i_X,2) MinColl{i_MF}(i_X,3)]);
        end
    end
end
% How to find the 'most sensitive time window'? The window with most effect
% WILL depend on the value of the MF. Furthermore, for different elements
% of the sample, the most-sensitive window will be somewhere else.
% Furthermore, it will differ between endpoints.

%% Return the globals to their original states

glo = glo_rem; % return glo to its original value

