function convert_deep(varargin)

%  Syntax: convert_deeep(varargin)
% 
% This function converts a MAT file generated after a DEBtox2019
% calibration (in BYOM) to the input format (CSV) for DeEP-tox on-line
% calculation tool: http://deep-tox.info/tool.html. When no filename is
% entered, the Matlab GUI element is used to select a MAT file. When the
% MAT file contains multiple sets of the same parameter, a dialog is
% presented so the user can select one. 
% 
% This function can be called from a byom_... script file (using glo.basenm
% or glo.mat_nm as input), or called from the command line (as long as the
% engine folders are in the path), or double-clicked in the command window.
% This function does not use any of the other BYOM code or settings. 
% 
% Author     : Tjalling Jager 
% Date       : June 2023
% Web support: http://www.debtox.info/byom.html and http://deep-tox.info/

%  Copyright (c) 2012-2024, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

%% Use input or Matlab GUI to select MAT file for conversion

warning('off','backtrace') % don't show entire trace when a warning is produced

if isempty(varargin) % if there is no input given, use the GUI element

    [fname_sample,filepath] = uigetfile('*.mat','Select a DEBtox2019 MAT file for conversion to DeEP format','MultiSelect','off');
    if ~iscell(fname_sample) && numel(fname_sample) == 1 && fname_sample(1) == 0 % if cancel is pressed ...
        disp('Cancel is pressed, so no MAT file has been loaded.')
        warning('on','backtrace'); return % simply stop
    end

    mat_nm = fname_sample(1:end-7); % remove the last part (e.g., _PS.mat) of the filename so that the basename remains
    % Note: if we ever want to load the sample with load_rnd, we might need to
    % place this in the global structure glo as well.

    % if the MAT file is not located in the current working directory, return the entire path
    if ~strcmp(filepath(1:end-1),pwd)
        disp(['   in directory: ',filepath])
        fname_load = [filepath,fname_sample];
    else
        fname_load = fname_sample;
    end

else % the filename is in the input
    % However, we don't know whether it is the 'base name' (as used in
    % glo.basenm and glo.mat_nm) or the full file name, with extension.
    % Both are valid as input. When it is NOT a full name, the code below
    % assumes that we have a MAT file generated by the parameter-space
    % explorer (so with filename ending on _PS.mat).

    fname_input = varargin{1};
    if strcmp(fname_input(end-3:end),'.mat') % then it is a full filename
        mat_nm     = fname_input(1:end-7); % remove the last part (e.g., _PS.mat) of the filename so that the basename remains
        fname_load = fname_input;
    else % assume it is a 'base name', and assume it is referring to a file made by the parameter-space explorer!
        mat_nm     = fname_input;
        fname_load = [fname_input,'_PS.mat'];        
    end
    if exist(fname_load,'file') ~= 2
        warning(['I am looking for the file ',fname_load,', but it cannot be found.'])
        warning('on','backtrace'); return % simply stop
    end
    fname_sample = fname_load; 
end

%% Display what we load and check if all necessary things are there
% At this moment, the MAT file is only used to obtain the parameter
% estimates and the settings in GLO. The sample itself is not used. In the
% future, the DeEP tool could be extended to propagate uncertainties from
% the calibration to the model prediction. This would require reading the
% sample from the MAT file, reducing it (as in load_rnd), and placing it in
% a text file.

disp('=================================================================================')
disp(['File selected for conversion: ',fname_sample])
type = fname_sample(end-5:end-4); 
if strcmp(type,'MC')
    disp('   Results from: Bayesian slice sampler')
elseif strcmp(type,'LR')
    disp('   Results from: likelihood-region shooting')
elseif strcmp(type,'PS')
    disp('   Results from: parameter-space explorer')
elseif strcmp(type,'LP') 
    disp('   MAT is for profile-likelihood results (does not include calibration results)')
    if exist([fname_sample(1:end-7),'_LR.mat'],'file') == 2
        disp('   the similarly named likelihood-region shooting file is available and can be used.')
    else
        error('Cannot continue without a MAT file with calibration results.')
    end
else
    warning('The selected MAT file is not of known type. No file generated.')
    warning('on','backtrace'); return % simply stop
end

% The MAT file must contain parameter structure par and the global settings in GLO.
listOfVariables = who('-file',fname_load); % read names of stored variables in mat file
if ismember('par', listOfVariables) % returns true if par is stored
    load(fname_load,'par') % read par if it is saved in the mat file!
    disp('   Loaded from saved MAT file, parameter structure par')
else
    warning('No parameter set is present in the saved MAT file! No file generated.')
    warning('on','backtrace'); return % simply stop
end
if ismember('GLO', listOfVariables) % returns true if GLO is stored
    load(fname_load,'GLO') % read GLO if it is saved in the mat file!
    disp('   Loaded from saved MAT file, global structure GLO')
else
    warning('No global structure GLO is present in the saved MAT file! No file generated.')
    warning('on','backtrace'); return % simply stop
end

% All required fields in par need to be available.
field_par1 = {'Lp','bb','kd','L0','Rm','zb','f','rB','Lm'};
if any(isfield(par,field_par1)==0)
    warning('The structure par in the MAT file does not contain all of the required fields. No file generated.')
    warning('on','backtrace'); return % simply stop
end

% But some should not be used (yet)
field_par2 = {'Lf','Lj','Tlag'};
for i = 1:length(field_par2)
    if isfield(par,field_par2{i}) && par.(field_par2{i})(1) > 0
        warning(['At the moment, use of parameter ',field_par2{i}, ' is not implemented into DeEP. No file generated.'])
        warning('on','backtrace'); return % simply stop
    end
end

% Some MAT files will contain more than one version of one or more
% parameters (to cover differences between simultaneously fitted data
% sets). Let the user select one.
if ~isempty(GLO.names_sep)
    warning('The MAT file contains separate parameters per data set (with glo.names_sep).')

    % first count how many parameters (hence different studies) there are
    i=1;
    list = {'1'};
    while isfield(par,[GLO.names_sep{1},num2str(i)])
        i = i + 1;
        list = [list,num2str(i)]; %#ok<AGROW>
    end

    % use a Matlab list dialog to allow the user to select a study
    [study,tf] = listdlg('PromptString',{'MAT file contains multiple', 'parameters',...
        'Select which study to use',''},'ListString',list,'SelectionMode','single');
    if tf == 0
        warning('No selection made. no file generated.')
        warning('on','backtrace'); return % simply stop
    end
    warning(['Parameters from data set ',num2str(study),' will be used to generate the output file.'])

    % copy the parameter for the selected study to the base position (this
    % is equivalent to what is done in call_deri).
    if study > 1 % when it is one, we don't need to do anything
        for i_sep = 1:length(GLO.names_sep) % run through extra parameter names for separate sets
            par.(GLO.names_sep{i_sep}) = par.([GLO.names_sep{i_sep},num2str(study-1)]); % copy extra parameter level to par
        end
    end

end

% All required fields in par and GLO need to be available.
field_glo = {'feedb','Lm_ref','yP','KRV','FBV','moa','kap'};
if any(isfield(GLO,field_glo)==0)
    warning('The structure GLO in the MAT file does not contain all of the required fields. No file generated.')
    warning('on','backtrace'); return % simply stop
end
if length(GLO.moa) < 5
    warning('The pMoA vector in GLO is not of correct length. No file generated.')
    warning('on','backtrace'); return % simply stop
end
if length(GLO.feedb) < 4
    warning('The feedbacks vector in GLO is not of correct length. No file generated.')
    warning('on','backtrace'); return % simply stop
end
if isfield(GLO,'Tbp') && GLO.Tbp > 0
    warning('At the moment, brood-pouch delay through glo.Tbp is not implemented in DeEP. No file generated.')
    warning('on','backtrace'); return % simply stop
end

disp('   All required fields in par and GLO are present in the MAT file')
disp('=================================================================================')
warning('on','backtrace')

%% Convert the information from the MAT file to the DeEP format
% DeEP uses a comma-separated text file.

BOOL = {'FALSE','TRUE'}; % DeEP uses strings rather than 0 or 1 for Booleans

fileID = fopen([mat_nm,'.csv'],'w'); % open file for writing
% The csv file gets the same name as the MAT file.

% Below, a long list of write statements. I don't see a simple way to do
% this in a more structured manner. The order is different from the file
% that DeEP saves, but order is not important.

% physiological parameters
fprintf(fileID,'%s,%0.10g\n','y0_1',par.L0(1)); % body length at start
fprintf(fileID,'%s,%0.10g\n','Lp',par.Lp(1));   % body length at puberty
fprintf(fileID,'%s,%0.10g\n','Lm',par.Lm(1));   % maximum body length
fprintf(fileID,'%s,%0.10g\n','rB',par.rB(1));   % von Bertalanffy growth rate constant
fprintf(fileID,'%s,%0.10g\n','Rm',par.Rm(1));   % maximum reproduction rate
fprintf(fileID,'%s,%0.10g\n','f',par.f(1));     % scaled food density

% toxicological parameters (sub-lethal effects only)
fprintf(fileID,'%s,%0.10g\n','kd',par.kd(1));
fprintf(fileID,'%s,%0.10g\n','z_b',par.zb(1));
fprintf(fileID,'%s,%0.10g\n','b_b',par.bb(1));

% physiological mode of action
fprintf(fileID,'%s,%s\n','SA',BOOL{1+GLO.moa(1)});
fprintf(fileID,'%s,%s\n','SM',BOOL{1+GLO.moa(2)});
fprintf(fileID,'%s,%s\n','SG',BOOL{1+GLO.moa(3)});
fprintf(fileID,'%s,%s\n','SR1',BOOL{1+GLO.moa(4)});
fprintf(fileID,'%s,%s\n','SR2',BOOL{1+GLO.moa(5)});

% feedbacks for damage dynamics
fprintf(fileID,'%s,%s\n','Xu',BOOL{1+GLO.feedb(1)});
fprintf(fileID,'%s,%s\n','Xe',BOOL{1+GLO.feedb(2)});
fprintf(fileID,'%s,%s\n','XG',BOOL{1+GLO.feedb(3)});
fprintf(fileID,'%s,%s\n','XR',BOOL{1+GLO.feedb(4)});

% global parameters
fprintf(fileID,'%s,%0.10g\n','F_BV',GLO.FBV);     % dry weight egg as fraction of structural body weight (for losses with repro)
fprintf(fileID,'%s,%0.10g\n','K_RV',GLO.KRV);     % part. coeff. repro buffer and structure (for losses with repro)
fprintf(fileID,'%s,%0.10g\n','kap',GLO.kap);      % approximation for kappa (for starvation response)
fprintf(fileID,'%s,%0.10g\n','yP',GLO.yP);        % product of yVA and yAV (for starvation response)
fprintf(fileID,'%s,%0.10g\n','Lm_TK',GLO.Lm_ref); % reference max length for scaling rate constants

% settings for the model predictions
fprintf(fileID,'%s,%0.10g\n','ECpct',10);    % default: calculate EP10
fprintf(fileID,'%s,%0.10g\n','interval',1);  % default: window interval of 1 day
fprintf(fileID,'%s,%s\n','thinning','TRUE'); % default: pruning windows that are unlikely to produce the lowest EPx
fprintf(fileID,'%s,%0.10g\n','windowLength',21); % default: window length of 21 days
fprintf(fileID,'%s,%s\n','detail','TRUE');   % default: detailed analysis on to calculate exact EPx
fprintf(fileID,'%s,%0.10g\n','passFail',10); % default: threshold for pass/fail of EPx set at 10
fprintf(fileID,'%s,%0.10g\n','cutOff',1000); % default: values of EPx above 1000 are set to 1000
fprintf(fileID,'%s,%0.10g\n','padDt',1);     % default: interval for windows before t=0 set to 1 day (just as 'interval')

% additional settings
fprintf(fileID,'%s,%s\n','Compound','to-fill-in'); % default: compound name is not in the MAT file
fprintf(fileID,'%s,%s\n','Species','to-fill-in');  % default: species name is not in the MAT file
fprintf(fileID,'%s,%s\n','Name',mat_nm);           % for now: use name of mat file
fprintf(fileID,'%s,%s\n','Model','standard');      % default: this field is currently not used

fclose(fileID); % close the file for writing

disp('CSV file generated, with the same name as the MAT file')
disp('=================================================================================')

